"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_js_1 = require("@solana/web3.js");
const common_1 = require("@project-serum/common");
const chai_1 = require("chai");
const chai_as_promised_1 = __importDefault(require("chai-as-promised"));
const src_1 = require("../src");
const metaplex_1 = require("./utils/metaplex");
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const amman_1 = require("@metaplex-foundation/amman");
(0, chai_1.use)(chai_as_promised_1.default);
describe('fanout', async () => {
    const connection = new web3_js_1.Connection(amman_1.LOCALHOST, 'confirmed');
    let authorityWallet;
    let fanoutSdk;
    beforeEach(async () => {
        authorityWallet = web3_js_1.Keypair.generate();
        await connection.requestAirdrop(authorityWallet.publicKey, web3_js_1.LAMPORTS_PER_SOL * 10);
        fanoutSdk = new src_1.FanoutClient(connection, new common_1.NodeWallet(new web3_js_1.Account(authorityWallet.secretKey)));
        await connection.requestAirdrop(authorityWallet.publicKey, web3_js_1.LAMPORTS_PER_SOL * 10);
    });
    describe('NFT Signing', () => {
        it('Can Sign As Creator', async () => {
            var _a, _b, _c, _d, _e, _f, _g, _h;
            const { fanout } = await fanoutSdk.initializeFanout({
                totalShares: 100,
                name: `Test${Date.now()}`,
                membershipModel: src_1.MembershipModel.NFT,
            });
            const fanoutAccount = await fanoutSdk.fetch(fanout, src_1.Fanout);
            const initMetadataData = new mpl_token_metadata_1.deprecated.DataV2({
                uri: 'URI',
                name: 'NAME',
                symbol: 'SYMBOL',
                sellerFeeBasisPoints: 1000,
                creators: [
                    new mpl_token_metadata_1.deprecated.Creator({
                        address: authorityWallet.publicKey.toBase58(),
                        share: 0,
                        verified: true,
                    }),
                    new mpl_token_metadata_1.deprecated.Creator({
                        address: fanoutAccount.accountKey.toBase58(),
                        share: 100,
                        verified: false,
                    }),
                ],
                collection: null,
                uses: null,
            });
            const nft = await (0, metaplex_1.createMasterEdition)(connection, authorityWallet, initMetadataData, 0);
            const sign = await fanoutSdk.signMetadata({
                fanout: fanout,
                metadata: nft.metadata,
            });
            const meta = await mpl_token_metadata_1.deprecated.Metadata.findByMint(connection, nft.mint.publicKey);
            (0, chai_1.expect)((_d = (_c = (_b = (_a = meta === null || meta === void 0 ? void 0 : meta.data) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.creators) === null || _c === void 0 ? void 0 : _c.at(1)) === null || _d === void 0 ? void 0 : _d.verified).to.equal(1);
            (0, chai_1.expect)((_h = (_g = (_f = (_e = meta === null || meta === void 0 ? void 0 : meta.data) === null || _e === void 0 ? void 0 : _e.data) === null || _f === void 0 ? void 0 : _f.creators) === null || _g === void 0 ? void 0 : _g.at(1)) === null || _h === void 0 ? void 0 : _h.address).to.equal(fanoutAccount.accountKey.toBase58());
        });
    });
});
//# sourceMappingURL=sign.js.map