"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.builtNftFanout = exports.builtWalletFanout = exports.builtTokenFanout = void 0;
const src_1 = require("../../src");
const metaplex_1 = require("./metaplex");
const mpl_token_metadata_1 = require("@metaplex-foundation/mpl-token-metadata");
const web3_js_1 = require("@solana/web3.js");
async function builtTokenFanout(mint, mintAuth, fanoutSdk, shares, numberMembers) {
    const name = `Test${Date.now()}`;
    const { fanout } = await fanoutSdk.initializeFanout({
        totalShares: 0,
        name: `Test${Date.now()}`,
        membershipModel: src_1.MembershipModel.Token,
        mint: mint.publicKey,
    });
    const mintInfo = await mint.getMintInfo();
    const totalSupply = shares ** mintInfo.decimals;
    const memberNumber = totalSupply / numberMembers;
    const members = [];
    for (let i = 0; i < numberMembers; i++) {
        const memberWallet = new web3_js_1.Keypair();
        await fanoutSdk.connection.requestAirdrop(memberWallet.publicKey, 10000000000);
        const ata = await mint.createAssociatedTokenAccount(memberWallet.publicKey);
        await mint.mintTo(ata, mintAuth, [], memberNumber);
        const ix = await fanoutSdk.stakeTokenMemberInstructions({
            shares: memberNumber,
            fanout: fanout,
            membershipMintTokenAccount: ata,
            membershipMint: mint.publicKey,
            member: memberWallet.publicKey,
            payer: memberWallet.publicKey,
        });
        console.log();
        const tx = await fanoutSdk.sendInstructions(ix.instructions, [memberWallet], memberWallet.publicKey);
        if (!!tx.RpcResponseAndContext.value.err) {
            const txdetails = await fanoutSdk.connection.getConfirmedTransaction(tx.TransactionSignature);
            console.log(txdetails, tx.RpcResponseAndContext.value.err);
        }
        members.push({
            voucher: ix.output.membershipVoucher,
            stakeAccount: ix.output.stakeAccount,
            wallet: memberWallet,
        });
    }
    const fanoutAccount = await fanoutSdk.fetch(fanout, src_1.Fanout);
    return {
        fanout: fanout,
        name,
        membershipMint: mint,
        fanoutAccountData: fanoutAccount,
        members: members,
    };
}
exports.builtTokenFanout = builtTokenFanout;
async function builtWalletFanout(fanoutSdk, shares, numberMembers) {
    const name = `Test${Date.now()}`;
    const init = await fanoutSdk.initializeFanout({
        totalShares: shares,
        name,
        membershipModel: src_1.MembershipModel.Wallet,
    });
    const memberNumber = shares / numberMembers;
    const ixs = [];
    const members = [];
    for (let i = 0; i < numberMembers; i++) {
        const memberWallet = new web3_js_1.Keypair();
        const ix = await fanoutSdk.addMemberWalletInstructions({
            fanout: init.fanout,
            fanoutNativeAccount: init.nativeAccount,
            membershipKey: memberWallet.publicKey,
            shares: memberNumber,
        });
        members.push({
            voucher: ix.output.membershipAccount,
            wallet: memberWallet,
        });
        ixs.push(...ix.instructions);
    }
    const tx = await fanoutSdk.sendInstructions(ixs, [], fanoutSdk.wallet.publicKey);
    if (!!tx.RpcResponseAndContext.value.err) {
        const txdetails = await fanoutSdk.connection.getConfirmedTransaction(tx.TransactionSignature);
        console.log(txdetails, tx.RpcResponseAndContext.value.err);
    }
    const fanoutAccount = await fanoutSdk.fetch(init.fanout, src_1.Fanout);
    return {
        fanout: init.fanout,
        name,
        fanoutAccountData: fanoutAccount,
        members: members,
    };
}
exports.builtWalletFanout = builtWalletFanout;
async function builtNftFanout(fanoutSdk, shares, numberMembers) {
    const name = `Test${Date.now()}`;
    const init = await fanoutSdk.initializeFanout({
        totalShares: shares,
        name,
        membershipModel: src_1.MembershipModel.NFT,
    });
    const memberNumber = shares / numberMembers;
    const ixs = [];
    const members = [];
    for (let i = 0; i < numberMembers; i++) {
        const memberWallet = new web3_js_1.Keypair();
        const initMetadataData = new mpl_token_metadata_1.deprecated.DataV2({
            uri: 'URI' + i,
            name: 'NAME' + i,
            symbol: 'SYMBOL' + i,
            sellerFeeBasisPoints: 1000,
            creators: null,
            collection: null,
            uses: null,
        });
        const nft = await (0, metaplex_1.createMasterEdition)(fanoutSdk.connection, fanoutSdk.wallet.payer, initMetadataData, 0);
        const tokenAccount = await nft.mint.getOrCreateAssociatedAccountInfo(memberWallet.publicKey);
        const owner = await nft.mint.getOrCreateAssociatedAccountInfo(fanoutSdk.wallet.publicKey);
        await nft.mint.transfer(owner.address, tokenAccount.address, fanoutSdk.wallet.payer, [], 1);
        const ix = await fanoutSdk.addMemberNftInstructions({
            fanout: init.fanout,
            fanoutNativeAccount: init.nativeAccount,
            membershipKey: nft.mint.publicKey,
            shares: memberNumber,
        });
        members.push({
            voucher: ix.output.membershipAccount,
            mint: nft.mint.publicKey,
            wallet: memberWallet,
        });
        ixs.push(...ix.instructions);
    }
    const tx = await fanoutSdk.sendInstructions(ixs, [], fanoutSdk.wallet.publicKey);
    if (!!tx.RpcResponseAndContext.value.err) {
        const txdetails = await fanoutSdk.connection.getConfirmedTransaction(tx.TransactionSignature);
        console.log(txdetails, tx.RpcResponseAndContext.value.err);
    }
    const fanoutAccount = await fanoutSdk.fetch(init.fanout, src_1.Fanout);
    return {
        fanout: init.fanout,
        name,
        fanoutAccountData: fanoutAccount,
        members: members,
    };
}
exports.builtNftFanout = builtNftFanout;
//# sourceMappingURL=scenarios.js.map