"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenUtils = void 0;
const spl_token_1 = require("@solana/spl-token");
const web3_js_1 = require("@solana/web3.js");
const chai_1 = require("chai");
class TokenUtils {
    constructor(provider) {
        this.provider = provider;
    }
    async expectBalance(account, balance) {
        const actual = await this.provider.connection.getTokenAccountBalance(account);
        (0, chai_1.expect)(actual.value.uiAmount).to.equal(balance);
    }
    async expectBalanceWithin(account, balance, precision) {
        const actual = await this.provider.connection.getTokenAccountBalance(account);
        (0, chai_1.expect)(actual.value.uiAmount).to.within(balance, precision);
    }
    async expectAtaBalance(account, mint, balance) {
        const ata = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, account);
        return this.expectBalance(ata, balance);
    }
    async createWrappedNativeAccount(provider, amount, wallet) {
        const newAccount = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, spl_token_1.NATIVE_MINT, wallet);
        const transaction = new web3_js_1.Transaction();
        if (!(await provider.connection.getAccountInfo(newAccount))) {
            transaction.add(spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, spl_token_1.NATIVE_MINT, newAccount, wallet, wallet));
        }
        transaction.add(web3_js_1.SystemProgram.transfer({
            fromPubkey: wallet,
            toPubkey: newAccount,
            lamports: amount,
        }));
        await provider.send(transaction);
        return newAccount;
    }
    async mintTo(mint, amount, destination, wallet) {
        const mintTx = new web3_js_1.Transaction();
        mintTx.add(spl_token_1.Token.createMintToInstruction(spl_token_1.TOKEN_PROGRAM_ID, mint, destination, wallet, [], amount));
        await this.provider.send(mintTx);
    }
    async sendTokens(provider, mint, to, amount, owner, payer) {
        const source = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, owner);
        const ata = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, to);
        const tx = new web3_js_1.Transaction({ feePayer: payer });
        if (!(await provider.connection.getAccountInfo(ata))) {
            tx.add(spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, ata, to, payer));
        }
        tx.add(spl_token_1.Token.createTransferInstruction(spl_token_1.TOKEN_PROGRAM_ID, source, ata, owner, [], amount));
        await provider.send(tx);
        return ata;
    }
    async createAtaAndMint(provider, mint, amount, to, authority, payer) {
        const ata = await spl_token_1.Token.getAssociatedTokenAddress(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, to);
        const mintTx = new web3_js_1.Transaction({ feePayer: payer });
        if (!(await provider.connection.getAccountInfo(ata))) {
            mintTx.add(spl_token_1.Token.createAssociatedTokenAccountInstruction(spl_token_1.ASSOCIATED_TOKEN_PROGRAM_ID, spl_token_1.TOKEN_PROGRAM_ID, mint, ata, to, payer));
        }
        mintTx.add(spl_token_1.Token.createMintToInstruction(spl_token_1.TOKEN_PROGRAM_ID, mint, ata, authority, [], amount));
        await provider.send(mintTx);
        return ata;
    }
}
exports.TokenUtils = TokenUtils;
//# sourceMappingURL=token.js.map